//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
// $Id$
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo...... 
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "ComplexGeometry.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Trd.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4NistManager.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4SystemOfUnits.hh"
#include "G4ios.hh"
 
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
ComplexGeometry::ComplexGeometry()
  :solidWorld(0),  logicWorld(0),  physiWorld(0),
   solidTarget(0), logicTarget(0), physiTarget(0), 
   solidBoxDetector(0), logicBoxDetector(0), physiBoxDetector(0), 
   solidTubsDetector(0), logicTubsDetector(0), physiTubsDetector(0), 
   solidTrapezoidDetector(0), logicTrapezoidDetector(0), physiTrapezoidDetector(0), 
   TargetMater(0), DetectorMater(0)
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
ComplexGeometry::~ComplexGeometry()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
G4VPhysicalVolume* ComplexGeometry::Construct()
{
//--------- Material definition ---------

  G4double a, z;
  G4double density;
  G4int nel;

  // Air -> deinition using G4Elements
  G4Element* N = new G4Element("Nitrogen", "N", z=7., a=14.01*CLHEP::g/mole);
  G4Element* O = new G4Element("Oxygen"  , "O", z=8., a=16.00*CLHEP::g/mole);
   
  G4Material* Air = new G4Material("Air", density=1.29*CLHEP::mg/cm3, nel=2);
  Air->AddElement(N, 70*CLHEP::perCent);
  Air->AddElement(O, 30*CLHEP::perCent);

  // Lead -> definition using G4Material only
  G4Material* Pb = new G4Material("Lead", z=82., a=207.19*CLHEP::g/mole, density=11.35*CLHEP::g/cm3);
  
  // define target material as lead
  TargetMater  = Pb;
 

  //  BGO scintillator -> definition using the GEANT4 library definition
  G4NistManager* man = G4NistManager::Instance();
  // define detector material as BGO
  DetectorMater = man->FindOrBuildMaterial("G4_BGO");


  // Print all the materials defined.
  G4cout << G4endl << "The materials defined are : " << G4endl << G4endl;
  G4cout << *(G4Material::GetMaterialTable()) << G4endl;

  


   

//--------- Definitions of Solids, Logical Volumes, Physical Volumes ---------
  
  //------------------------------ 
  // World
  //------------------------------ 
 
  const G4double HalfWorldSize = 100.*CLHEP::cm;

  solidWorld= new G4Box("World",HalfWorldSize, HalfWorldSize, HalfWorldSize);
  logicWorld= new G4LogicalVolume(solidWorld, Air, "World", 0, 0, 0);
  
  //  Must place the World Physical volume unrotated at (0,0,0).
  // 
  physiWorld = new G4PVPlacement(0,               // no rotation
                                 G4ThreeVector(), // at (0,0,0)
                                 logicWorld,      // its logical volume
                                 "World",         // its name
                                 0,               // its mother  volume
                                 false,           // no boolean operations
                                 0);              // copy number
                                 
  //------------------------------ 
  // Target
  //------------------------------
  
  G4double targetSize = 5.*CLHEP::cm;
  G4ThreeVector positionTarget = G4ThreeVector(0., 0., 0.);
   
  solidTarget = new G4Box("Target", targetSize, targetSize, targetSize);
  logicTarget = new G4LogicalVolume(solidTarget, TargetMater, "Target", 0, 0, 0);
  physiTarget = new G4PVPlacement(0,               // no rotation
                                  positionTarget,  // at (x,y,z)
                                  logicTarget,     // its logical volume                                  
                                  "Target",        // its name
                                  logicWorld,      // its mother  volume
                                  false,           // no boolean operations
                                  0);              // copy number 



  //------------------------------ 
  // Detectors
  //------------------------------
  

  const G4double distanceToDetectors = 60.;



  //######################################
  //############ BOX DETECTOR ############
  //######################################
   

  G4ThreeVector positionBoxDetector = G4ThreeVector(0., 0., distanceToDetectors*CLHEP::cm);
  const G4double boxDetectorSize = 5.*CLHEP::cm;  
  const G4double boxDetectorThickness = 2.5*CLHEP::cm;

  solidBoxDetector = new G4Box("BoxDetector", boxDetectorSize, boxDetectorSize, boxDetectorThickness);
  logicBoxDetector = new G4LogicalVolume(solidBoxDetector, DetectorMater, "BoxDetector", 0, 0, 0);  
  physiBoxDetector = new G4PVPlacement(0,              // no rotation
				       positionBoxDetector, // at (x,y,z)
				       logicBoxDetector,    // its logical volume                                  
				       "BoxDetector",       // its name
				       logicWorld,      // its mother  volume
				       false,           // no boolean operations
				       1);              // copy number 


  
  
  //#####################################
  //########### TUBS DETECTOR ###########
  //#####################################

  // define dimensions
  const G4double innerRadius = 0.*CLHEP::cm;
  const G4double outerRadius = 5.*CLHEP::cm;
  const G4double tubsLength = 10.*CLHEP::cm;
  const G4double startingAngle = 0.*CLHEP::deg;
  const G4double spanningAngle = 360.*CLHEP::deg;

  // define angles to position the detector
  const double tubsAngle = 60.;
  const double tubsAngleRad = tubsAngle*CLHEP::pi/180.;

  // calculate position based on the angle
  const G4double tubsPosX = distanceToDetectors*std::sin(tubsAngleRad)*cm;
  const G4double tubsPosY = 0.*CLHEP::cm;
  const G4double tubsPosZ = distanceToDetectors*std::cos(tubsAngleRad)*cm;
  G4ThreeVector positionTubsDetector = G4ThreeVector(tubsPosX, tubsPosY, tubsPosZ);
  
  // define the rotation of the detector in order to have the detector facing the target
  G4RotationMatrix rotTubs;
  rotTubs.rotateY(tubsAngle*CLHEP::deg);


  solidTubsDetector = new G4Tubs("TubsDetector", innerRadius, outerRadius, tubsLength, startingAngle, spanningAngle);
  logicTubsDetector = new G4LogicalVolume(solidTubsDetector, DetectorMater, "TubsDetector", 0, 0, 0);  
  physiTubsDetector = new G4PVPlacement(G4Transform3D(rotTubs, positionTubsDetector),              // with rotation and translation		
					logicTubsDetector,    // its logical volume                                  
					"TubsDetector",       // its name
					logicWorld,      // its mother  volume
					false,           // no boolean operations
					2);              // copy number 



  //##########################################
  //########### TRAPEZOID DETECTOR ###########
  //##########################################
  
  // define dimensions
  const G4double halfTargetFacetX = 2.5*CLHEP::cm;
  const G4double halfTargetFacetY = 2.5*CLHEP::cm;
  const G4double halfBaseFacetX = 7.5*CLHEP::cm;
  const G4double halfBaseFacetY = 7.5*CLHEP::cm;
  const G4double halfTrapezoidLength = 10.*CLHEP::cm;

  // define angles to position the detector
  const double trapezoidAngle = -60.;
  const double trapezoidAngleRad = trapezoidAngle*CLHEP::pi/180.;

  // calculate position based on the angle
  const G4double trapezoidPosX = distanceToDetectors*std::sin(trapezoidAngleRad)*cm;
  const G4double trapezoidPosY = 0.*cm;
  const G4double trapezoidPosZ = distanceToDetectors*std::cos(trapezoidAngleRad)*cm;
  G4ThreeVector positionTrapezoidDetector = G4ThreeVector(trapezoidPosX, trapezoidPosY, trapezoidPosZ);
  
  // define the rotation of the detector in order to have the detector facing the target
  G4RotationMatrix rotTrapezoid;
  rotTrapezoid.rotateY(trapezoidAngle*CLHEP::deg);

  solidTrapezoidDetector = new G4Trd("TrapezoidDetector", halfTargetFacetX, halfBaseFacetX, halfTargetFacetY, halfBaseFacetY, halfTrapezoidLength);
  logicTrapezoidDetector = new G4LogicalVolume(solidTrapezoidDetector, DetectorMater, "TrapezoidDetector", 0, 0, 0);  
  physiTrapezoidDetector = new G4PVPlacement(G4Transform3D(rotTrapezoid, positionTrapezoidDetector),              // with rotation and translation		
					     logicTrapezoidDetector,    // its logical volume                                  
					     "TrapezoidDetector",       // its name
					     logicWorld,                // its mother  volume
					     false,                     // no boolean operations
					     3);                        // copy number 
  



 
  return physiWorld;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
 
